@@ -177,8 +177,11 @@ class Expr:
                 if isinstance(v, functools.cached_property) and k in self.__dict__:
                     cache[k] = getattr(self, k)
 
-        return Expr._reconstruct, tuple(
-            [type(self), *self.operands, self.deterministic_token, cache]
+        return Expr._reconstruct, (
+            type(self),
+            *self.operands,
+            self.deterministic_token,
+            cache,
         )
 
     def _depth(self, cache=None):
@@ -304,7 +307,7 @@ class Expr:
         expr:
             output expression
         changed:
-            whether or not any change occured
+            whether or not any change occurred
         """
         if self._name in rewritten:
             return rewritten[self._name]
@@ -463,7 +466,7 @@ class Expr:
         return
 
     def lower_once(self, lowered: dict):
-        # Check for a chached result
+        # Check for a cached result
         try:
             return lowered[self._name]
         except KeyError:
