@@ -601,10 +601,11 @@ def take(outname, inname, chunks, index, axis=0):
 
     if not np.isnan(chunks[axis]).any():
         from dask.array._shuffle import _shuffle
-        from dask.array.utils import arange_safe, asarray_safe
+        from dask.array.utils import asarray_safe
 
-        arange = arange_safe(np.sum(chunks[axis]), like=index)
-        if len(index) == len(arange) and np.abs(index - arange).sum() == 0:
+        # verify if this is a full arange (the equivalent of `slice(None)`)
+        full_length = sum(chunks[axis])
+        if len(index) == full_length and index[0] == 0 and np.all(np.diff(index) == 1):
             # TODO: This should be a real no-op, but the call stack is
             # too deep to do this efficiently for now
             chunk_tuples = product(*(range(len(c)) for i, c in enumerate(chunks)))
@@ -614,7 +615,7 @@ def take(outname, inname, chunks, index, axis=0):
             }
             return tuple(chunks), graph
 
-        average_chunk_size = int(sum(chunks[axis]) / len(chunks[axis]))
+        average_chunk_size = int(full_length / len(chunks[axis]))
 
         indexer = []
         index = asarray_safe(index, like=index)
@@ -2084,7 +2085,7 @@ def setitem(x, v, indices):
     >>> y is x
     True
     """
-    if not v.size:
+    if not math.prod(v.shape):
         return x
 
     # Normalize integer array indices
