@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import re
+import textwrap
 import warnings
 from collections.abc import Iterable
 
@@ -27,23 +28,11 @@ from dask.dataframe.utils import (
     meta_series_constructor,
     pyarrow_strings_enabled,
     raise_on_meta_error,
-    shard_df_on_index,
     valid_divisions,
 )
 from dask.local import get_sync
 
 
-def test_shard_df_on_index():
-    df = pd.DataFrame(
-        {"x": [1, 2, 3, 4, 5, 6], "y": list("abdabd")}, index=[10, 20, 30, 40, 50, 60]
-    )
-
-    result = list(shard_df_on_index(df, [20, 50]))
-    assert list(result[0].index) == [10]
-    assert list(result[1].index) == [20, 30, 40]
-    assert list(result[2].index) == [50, 60]
-
-
 def test_make_meta():
     df = pd.DataFrame(
         {"a": [1, 2, 3], "b": list("abc"), "c": [1.0, 2.0, 3.0]}, index=[10, 20, 30]
@@ -56,12 +45,19 @@ def test_make_meta():
     assert isinstance(meta.index, type(df.index))
     # - ensure no references to original data arrays are kept
     for col in "abc":
-        meta_pointer = meta[col].values.__array_interface__["data"][0]
-        df_pointer = df[col].values.__array_interface__["data"][0]
+        if PANDAS_GE_300 and col == "b":
+            # backed by an arrow array
+            meta_pointer = meta[col].array._pa_array.chunks[0].buffers()[1].address
+            df_pointer = df[col].array._pa_array.chunks[0].buffers()[1].address
+        else:
+            # backed by a numpy array
+            meta_pointer = meta[col].values.__array_interface__["data"][0]
+            df_pointer = df[col].values.__array_interface__["data"][0]
         assert meta_pointer != df_pointer
-    meta_pointer = meta.index.values.__array_interface__["data"][0]
-    df_pointer = df.index.values.__array_interface__["data"][0]
-    assert meta_pointer != df_pointer
+
+        meta_index_pointer = meta.index.values.__array_interface__["data"][0]
+        df_index_pointer = df.index.values.__array_interface__["data"][0]
+        assert meta_index_pointer != df_index_pointer
 
     # Pandas series
     meta = make_meta(df.a)
@@ -90,14 +86,22 @@ def test_make_meta():
     assert make_meta(ddf) is ddf._meta
 
     # Dict
-    meta = make_meta({"a": "i8", "b": "O", "c": "f8"})
+    if PANDAS_GE_300:
+        meta = make_meta({"a": "i8", "b": "str", "c": "f8"})
+    else:
+        meta = make_meta({"a": "i8", "b": "O", "c": "f8"})
+
     assert isinstance(meta, pd.DataFrame)
     assert len(meta) == 0
     assert (meta.dtypes == df.dtypes).all()
     assert isinstance(meta.index, pd.RangeIndex)
 
     # List
-    meta = make_meta([("a", "i8"), ("c", "f8"), ("b", "O")])
+    if PANDAS_GE_300:
+        meta = make_meta([("a", "i8"), ("c", "f8"), ("b", "str")])
+    else:
+        meta = make_meta([("a", "i8"), ("c", "f8"), ("b", "O")])
+
     assert (meta.columns == ["a", "c", "b"]).all()
     assert len(meta) == 0
     assert (meta.dtypes == df.dtypes[meta.dtypes.index]).all()
@@ -115,7 +119,10 @@ def test_make_meta():
         """Custom class iterator returning pandas types."""
 
         def __init__(self, max=0):
-            self.types = [("a", "i8"), ("c", "f8"), ("b", "O")]
+            if PANDAS_GE_300:
+                self.types = [("a", "i8"), ("c", "f8"), ("b", "str")]
+            else:
+                self.types = [("a", "i8"), ("c", "f8"), ("b", "O")]
 
         def __iter__(self):
             self.n = 0
@@ -213,8 +220,13 @@ def test_meta_nonempty():
     df3 = meta_nonempty(df2)
     assert (df3.dtypes == df2.dtypes).all()
     assert df3["A"][0] == "Alice"
-    assert df3["B"][0] == "foo"
-    assert df3["C"][0] == "foo"
+
+    if PANDAS_GE_300:
+        assert df3["B"][0] == "a"
+        assert df3["C"][0] == "a"
+    else:
+        assert df3["B"][0] == "foo"
+        assert df3["C"][0] == "foo"
     assert df3["D"][0] == np.float32(1)
     assert df3["D"][0].dtype == "f4"
     assert df3["E"][0] == np.int32(1)
@@ -222,7 +234,10 @@ def test_meta_nonempty():
     assert df3["F"][0] == pd.Timestamp("1970-01-01 00:00:00")
     assert df3["G"][0] == pd.Timestamp("1970-01-01 00:00:00", tz="America/New_York")
     assert df3["H"][0] == pd.Timedelta("1")
-    assert df3["I"][0] == "foo"
+    if PANDAS_GE_300:
+        assert type(df3["I"][0]) is object
+    else:
+        assert df3["I"][0] == "foo"
     assert df3["J"][0] == UNKNOWN_CATEGORIES
     assert len(df3["K"].cat.categories) == 0
 
@@ -235,8 +250,13 @@ def test_meta_duplicated():
     df = pd.DataFrame(columns=["A", "A", "B"])
     res = meta_nonempty(df)
 
+    if PANDAS_GE_300:
+        o = dd.utils._object
+    else:
+        o = "foo"
+
     exp = pd.DataFrame(
-        [["foo", "foo", "foo"], ["foo", "foo", "foo"]],
+        [[o, o, o], [o, o, o]],
         index=meta_nonempty(df.index),
         columns=["A", "A", "B"],
     )
@@ -430,35 +450,46 @@ def test_check_meta():
         check_meta(df2, meta2, funcname="from_delayed")
     frame = "pandas.core.frame.DataFrame" if not PANDAS_GE_300 else "pandas.DataFrame"
 
-    exp = (
-        "Metadata mismatch found in `from_delayed`.\n"
-        "\n"
-        f"Partition type: `{frame}`\n"
-        "+--------+----------+----------+\n"
-        "| Column | Found    | Expected |\n"
-        "+--------+----------+----------+\n"
-        "| 'a'    | object   | category |\n"
-        "| 'c'    | -        | float64  |\n"
-        "| 'e'    | category | -        |\n"
-        "+--------+----------+----------+"
+    if PANDAS_GE_300:
+        string_type = "str   "  # space for alignment
+    else:
+        string_type = "object"
+
+    exp = textwrap.dedent(
+        f"""\
+        Metadata mismatch found in `from_delayed`.
+
+        Partition type: `{frame}`
+        +--------+----------+----------+
+        | Column | Found    | Expected |
+        +--------+----------+----------+
+        | 'a'    | {string_type}   | category |
+        | 'c'    | -        | float64  |
+        | 'e'    | category | -        |
+        +--------+----------+----------+"""
     )
+
     assert str(err.value) == exp
 
     # pandas dtype metadata error
     with pytest.raises(ValueError) as err:
         check_meta(df.a, pd.Series([], dtype="string"), numeric_equal=False)
-    assert str(err.value) == (
-        "Metadata mismatch found.\n"
-        "\n"
-        f"Partition type: `{series}`\n"
-        "+----------+--------+\n"
-        "|          | dtype  |\n"
-        "+----------+--------+\n"
-        "| Found    | object |\n"
-        "| Expected | string |\n"
-        "+----------+--------+"
+
+    expected = textwrap.dedent(
+        f"""\
+        Metadata mismatch found.
+
+        Partition type: `{series}`
+        +----------+--------+
+        |          | dtype  |
+        +----------+--------+
+        | Found    | {string_type} |
+        | Expected | string |
+        +----------+--------+"""
     )
 
+    assert str(err.value) == expected
+
 
 def test_check_matching_columns_raises_appropriate_errors():
     df = pd.DataFrame(columns=["a", "b", "c"])
